﻿using System;
using System.Collections.Generic;
using Grasshopper.Kernel;
using Grasshopper.Kernel.Types;
using Grasshopper.Kernel.Data;
using System.Linq;
using Dlubal.RFEM5;
using System.Runtime.InteropServices;

namespace rFER_ExcelExp
{
    public class rFEM_3D_Export : GH_Component
    {
        /// <summary>
        /// Each implementation of GH_Component must provide a public 
        /// constructor without any arguments.
        /// Category represents the Tab in which the component will appear, 
        /// Subcategory the panel. If you use non-existing tab or panel names, 
        /// new tabs/panels will automatically be created.
        /// </summary>
        public rFEM_3D_Export()
          : base("rFEM 3D Export", "rFEM 3D Export",
              "rFEM 3D Export",
              "rFEM 3D Export", "rFEM 3D Export")
        {
        }

        /// <summary>
        /// Registers all the input parameters for this component.
        /// </summary>
        protected override void RegisterInputParams(GH_Component.GH_InputParamManager pManager)
        {
            pManager.AddPointParameter("KnotenListe", "Knoten Liste", "Knoten Liste", GH_ParamAccess.list);

            pManager.AddPointParameter("Seite A: Knoten oben", "Seite A: Knoten oben", "Seite A: Knoten oben", GH_ParamAccess.tree);
            pManager.AddTextParameter("Typ oben", "Typ oben", "z.B. NURBS, Bogen", GH_ParamAccess.item);

            pManager.AddPointParameter("Seite A: Knoten Mitte", "Seite A: Knoten Mitte", "Seite A: Knoten Mitte", GH_ParamAccess.tree);
            pManager.AddTextParameter("Typ Mitte", "Typ Mitte", "z.B. NURBS, Bogen", GH_ParamAccess.item);

            pManager.AddPointParameter("Seite A: Knoten quer", "Seite A: Knoten quer", "Seite A: Knoten quer", GH_ParamAccess.tree);
            pManager.AddTextParameter("Typ quer", "Typ quer", "z.B. NURBS, Bogen", GH_ParamAccess.item);

            pManager.AddPointParameter("Seite A: Knoten unten", "Seite A: Knoten unten", "Seite A: Knoten unten", GH_ParamAccess.tree);
            pManager.AddTextParameter("Typ unten", "Typ unten", "z.B. NURBS, Bogen", GH_ParamAccess.item);

            pManager.AddPointParameter("Verbindung oben", "Verbindung oben", "Verbindung oben", GH_ParamAccess.tree);
            pManager.AddTextParameter("Typ Verbindung", "Typ Verbindung", "z.B. NURBS, Bogen", GH_ParamAccess.item);

            pManager.AddPointParameter("Verbindung unten", "Verbindung unten", "Verbindung unten", GH_ParamAccess.tree);

            pManager.AddPointParameter("Seite B: Knoten oben", "Seite B: Knoten oben", "Seite B: Knoten oben", GH_ParamAccess.tree);

            pManager.AddPointParameter("Seite B: Knoten Mitte", "Seite B: Knoten Mitte", "Seite B: Knoten Mitte", GH_ParamAccess.tree);

            pManager.AddPointParameter("Seite B: Knoten quer", "Seite B: Knoten quer", "Seite B: Knoten quer", GH_ParamAccess.tree);

            pManager.AddPointParameter("Seite B: Knoten unten", "Seite B: Knoten unten", "Seite B: Knoten unten", GH_ParamAccess.tree);

            pManager.AddIntegerParameter("Anzahl pro Element", "Int", "Count", GH_ParamAccess.item);

            pManager.AddBooleanParameter("Connect to RFEM", "RFEM", "RFEM", GH_ParamAccess.item, false);
        }

        /// <summary>
        /// Registers all the output parameters for this component.
        /// </summary>
        protected override void RegisterOutputParams(GH_Component.GH_OutputParamManager pManager)
        {
            //pManager.AddTextParameter("test", "test", "testc", GH_ParamAccess.list);
            //pManager.AddTextParameter("Knoten Liste", "Knoten Liste", "Knoten Liste", GH_ParamAccess.list);
        }

        /// <summary>
        /// This is the method that actually does the work.
        /// </summary>
        /// <param name="DA">The DA object can be used to retrieve data from input parameters and 
        /// to store data in output parameters.</param>
        protected override void SolveInstance(IGH_DataAccess DA)
        {
            // ===============================================================================================
            // Read input parameters
            // ===============================================================================================

            Boolean rfemEX = false;

            List<GH_Point> kList = new List<GH_Point>();
            GH_Structure<GH_Point> pObenA = new GH_Structure<GH_Point>();
            GH_Structure<GH_Point> pMitteA = new GH_Structure<GH_Point>();
            GH_Structure<GH_Point> pQuerA = new GH_Structure<GH_Point>();
            GH_Structure<GH_Point> pUntenA = new GH_Structure<GH_Point>();
            GH_Structure<GH_Point> pVerbindungO = new GH_Structure<GH_Point>();
            GH_Structure<GH_Point> pVerbindungU = new GH_Structure<GH_Point>();

            GH_Structure<GH_Point> pObenB = new GH_Structure<GH_Point>();
            GH_Structure<GH_Point> pMitteB = new GH_Structure<GH_Point>();
            GH_Structure<GH_Point> pQuerB = new GH_Structure<GH_Point>();
            GH_Structure<GH_Point> pUntenB = new GH_Structure<GH_Point>();

            string TypOben = "";
            string TypMitte = "";
            string TypQuer = "";
            string TypUnten = "";
            string TypVerbindung = "";
            int abschnittAnzahl = 100;

            DA.GetDataList("KnotenListe", kList);
            DA.GetDataTree<GH_Point>("Seite A: Knoten oben", out pObenA);
            DA.GetDataTree<GH_Point>("Seite A: Knoten Mitte", out pMitteA);
            DA.GetDataTree<GH_Point>("Seite A: Knoten quer", out pQuerA);
            DA.GetDataTree<GH_Point>("Seite A: Knoten unten", out pUntenA);
            DA.GetDataTree<GH_Point>("Verbindung oben", out pVerbindungO);
            DA.GetDataTree<GH_Point>("Verbindung unten", out pVerbindungU);
            DA.GetDataTree<GH_Point>("Seite B: Knoten oben", out pObenB);
            DA.GetDataTree<GH_Point>("Seite B: Knoten Mitte", out pMitteB);
            DA.GetDataTree<GH_Point>("Seite B: Knoten quer", out pQuerB);
            DA.GetDataTree<GH_Point>("Seite B: Knoten unten", out pUntenB);
            
            DA.GetData("Typ oben", ref TypOben);
            DA.GetData("Typ Mitte", ref TypMitte);
            DA.GetData("Typ quer", ref TypQuer);
            DA.GetData("Typ unten", ref TypUnten);
            DA.GetData("Typ Verbindung", ref TypVerbindung);
            DA.GetData("Anzahl pro Element", ref abschnittAnzahl);
            DA.GetData("Connect to RFEM", ref rfemEX);

            // abschnittAnzahl = abschnittAnzahl - 1;

            if (rfemEX)
            {
                ///////// Seite A
                // Anzahl Vorhandener Objekte 
                IModel model = Marshal.GetActiveObject("RFEM5.Model") as IModel;
                model.GetApplication().LockLicense();
                IModelData data = model.GetModelData();
                int nodeCount = data.GetNodeCount();
                int lineCount = data.GetLineCount();
                int surfaceCount = data.GetSurfaceCount();
                int solidCount = data.GetSolidCount();
                model.GetApplication().UnlockLicense();

                // Export Knoten
                RFEM_Connect RFEM_COM = new RFEM_Connect();
                foreach (GH_Point node in kList) RFEM_COM.CreateNode(node);

                // Export Linien
                KnotenNrZuordnen lObenA = new KnotenNrZuordnen(kList, pObenA, nodeCount);
                KnotenNrZuordnen lMitteA = new KnotenNrZuordnen(kList, pMitteA, nodeCount);
                KnotenNrZuordnen lQuerA = new KnotenNrZuordnen(kList, pQuerA, nodeCount);
                KnotenNrZuordnen lUntenA = new KnotenNrZuordnen(kList, pUntenA, nodeCount);
                KnotenNrZuordnen lVerbindungO = new KnotenNrZuordnen(kList, pVerbindungO, nodeCount);
                KnotenNrZuordnen lVerbindungU = new KnotenNrZuordnen(kList, pVerbindungU, nodeCount);
                KnotenNrZuordnen lObenB = new KnotenNrZuordnen(kList, pObenB, nodeCount);
                KnotenNrZuordnen lMitteB = new KnotenNrZuordnen(kList, pMitteB, nodeCount);
                KnotenNrZuordnen lQuerB = new KnotenNrZuordnen(kList, pQuerB, nodeCount);
                KnotenNrZuordnen lUntenB = new KnotenNrZuordnen(kList, pUntenB, nodeCount);

                foreach (string e in lObenA.FindeKnotenNr()) RFEM_COM.CreateLine(e, TypOben, "L Oben A");
                foreach (string e in lMitteA.FindeKnotenNr()) RFEM_COM.CreateLine(e, TypMitte, "L Mitte A");
                foreach (string e in lQuerA.FindeKnotenNr()) RFEM_COM.CreateLine(e, TypQuer, "L Quer A");
                foreach (string e in lUntenA.FindeKnotenNr()) RFEM_COM.CreateLine(e, TypUnten, "L Unten A");
                foreach (string e in lVerbindungO.FindeKnotenNr()) RFEM_COM.CreateLine(e, TypVerbindung, "Verbindung O");
                foreach (string e in lVerbindungU.FindeKnotenNr()) RFEM_COM.CreateLine(e, TypVerbindung, "Verbindung U");
                foreach (string e in lObenB.FindeKnotenNr()) RFEM_COM.CreateLine(e, TypOben, "L Oben B");
                foreach (string e in lMitteB.FindeKnotenNr()) RFEM_COM.CreateLine(e, TypMitte, "L Mitte B");
                foreach (string e in lQuerB.FindeKnotenNr()) RFEM_COM.CreateLine(e, TypQuer, "L Quer B");
                foreach (string e in lUntenB.FindeKnotenNr()) RFEM_COM.CreateLine(e, TypUnten, "L Unten B");

                // Export Flaechen Seite A **
                List<string> LlistA = new List<string>();
                List<int> intOBJ = new List<int>();
                int z = 0;

                foreach (string e in lObenA.FindeKnotenNr())
                {
                    int off1 = lObenA.FindeKnotenNr().Count + lMitteA.FindeKnotenNr().Count + 1 + lineCount + (z/ abschnittAnzahl);
                    int off2 = off1 + 1;
                    int off3 = lObenA.FindeKnotenNr().Count + lMitteA.FindeKnotenNr().Count + 1 + lineCount + lQuerA.FindeKnotenNr().Count;
                    string lUmgrenzung = Convert.ToString(z + 1 + lineCount) + ", " + Convert.ToString(z + off1) + ", " + Convert.ToString(z + off2) + ", " + Convert.ToString(z + off3);
                    LlistA.Add(lUmgrenzung);
                    intOBJ.Add(lObenA.FindeKnotenNr().Count + z + 1 + lineCount);
                    z++;
                }

                foreach (string l in LlistA) { RFEM_COM.CreateQuad(l); }

                // Export Flaechen Oben **
                List<string> LlistO = new List<string>();
                z = 0;
                foreach (string e in lObenA.FindeKnotenNr())
                {
                    int off1 = lineCount + lObenA.FindeKnotenNr().Count + lMitteA.FindeKnotenNr().Count + lQuerA.FindeKnotenNr().Count + lUntenA.FindeKnotenNr().Count + (z / abschnittAnzahl);
                    int off2 = lineCount + lObenA.FindeKnotenNr().Count + lMitteA.FindeKnotenNr().Count + lQuerA.FindeKnotenNr().Count + lUntenA.FindeKnotenNr().Count + lVerbindungO.FindeKnotenNr().Count + lVerbindungU.FindeKnotenNr().Count;
                    string lUmgrenzung = Convert.ToString(z + 1 + lineCount) + ", " + Convert.ToString(z + off1 + 1 ) + ", " + Convert.ToString(z + off2 +1 ) + ", " + Convert.ToString(z + off1 + 2);
                    LlistO.Add(lUmgrenzung);
                    z++;
                }

                foreach (string l in LlistO) { RFEM_COM.CreateQuad(l); }

                // Export Flaechen Unten *
                List<string> LlistU = new List<string>();
                z = 0;
                foreach (string e in lObenA.FindeKnotenNr())
                {
                    int off1 = lineCount + lObenA.FindeKnotenNr().Count + lMitteA.FindeKnotenNr().Count + lQuerA.FindeKnotenNr().Count;
                    int off2 = lineCount + lObenA.FindeKnotenNr().Count + lMitteA.FindeKnotenNr().Count + lQuerA.FindeKnotenNr().Count + lUntenA.FindeKnotenNr().Count + lVerbindungO.FindeKnotenNr().Count + (z / abschnittAnzahl);
                    int off3 = lineCount + lObenA.FindeKnotenNr().Count + lMitteA.FindeKnotenNr().Count + lQuerA.FindeKnotenNr().Count + lUntenA.FindeKnotenNr().Count + lVerbindungO.FindeKnotenNr().Count + lVerbindungU.FindeKnotenNr().Count + lObenB.FindeKnotenNr().Count + lMitteB.FindeKnotenNr().Count + lQuerB.FindeKnotenNr().Count;
                    int off4 = off2 + 1;
                    string lUmgrenzung = Convert.ToString(z + 1 + off1) + ", " + Convert.ToString(z + 1 + off2) + ", " + Convert.ToString(z + 1 + off3) + ", " + Convert.ToString(z + 1 + off4);
                    LlistU.Add(lUmgrenzung);
                    z++;
                }

                foreach (string l in LlistU) { RFEM_COM.CreateQuad(l); }

                // Export Flaechen Zwischen *
                List<string> LlistZ = new List<string>();
                z = 0;

                foreach (string e in lQuerA.FindeKnotenNr())
                {
                    int off1 = lineCount + lObenA.FindeKnotenNr().Count + lMitteA.FindeKnotenNr().Count;
                    int off2 = lineCount + lObenA.FindeKnotenNr().Count + lMitteA.FindeKnotenNr().Count + lQuerA.FindeKnotenNr().Count + lUntenA.FindeKnotenNr().Count;
                    int off3 = lineCount + lObenA.FindeKnotenNr().Count + lMitteA.FindeKnotenNr().Count + lQuerA.FindeKnotenNr().Count + lUntenA.FindeKnotenNr().Count + lVerbindungO.FindeKnotenNr().Count + lVerbindungU.FindeKnotenNr().Count + lObenB.FindeKnotenNr().Count + lMitteB.FindeKnotenNr().Count;
                    int off4 = lineCount + lObenA.FindeKnotenNr().Count + lMitteA.FindeKnotenNr().Count + lQuerA.FindeKnotenNr().Count + lUntenA.FindeKnotenNr().Count + lVerbindungO.FindeKnotenNr().Count;
                    string lUmgrenzung = Convert.ToString(z + 1 + off1) + ", " + Convert.ToString(z + 1 + off2) + ", " + Convert.ToString(z + 1 + off3) + ", " + Convert.ToString(z + 1 + off4);
                    LlistZ.Add(lUmgrenzung);
                    z++;
                }
                foreach (string l in LlistZ) { RFEM_COM.CreateQuad(l); }
                // Export Flaechen Seite B *

                List<string> LlistB = new List<string>();
                List<int> intOBJ_B = new List<int>();
                z = 0;
                int offx = lineCount + lObenA.FindeKnotenNr().Count + lMitteA.FindeKnotenNr().Count + lQuerA.FindeKnotenNr().Count + lUntenA.FindeKnotenNr().Count + lVerbindungO.FindeKnotenNr().Count + lVerbindungU.FindeKnotenNr().Count; 
                foreach (string e in lObenB.FindeKnotenNr())
                {
                    /*
                    int off1 = lObenB.FindeKnotenNr().Count + lMitteB.FindeKnotenNr().Count + 1 + lineCount + + offx + (z / abschnittAnzahl);
                    int off2 = off1 + 1;
                    int off3 = lObenB.FindeKnotenNr().Count + lMitteB.FindeKnotenNr().Count + 1 + lineCount + lQuerB.FindeKnotenNr().Count;
                    */
                    int off1 = offx + lObenB.FindeKnotenNr().Count + lMitteB.FindeKnotenNr().Count + (z / abschnittAnzahl);
                    int off2 = off1 + 1;
                    int off3 = offx + lObenB.FindeKnotenNr().Count + lMitteB.FindeKnotenNr().Count + lQuerB.FindeKnotenNr().Count;
                    string lUmgrenzung = Convert.ToString(z + 1 + lineCount + offx) + ", " + Convert.ToString(z + 1+ off1) + ", " + Convert.ToString(z + 1 + off2) + ", " + Convert.ToString(z + 1 + off3);
                    LlistB.Add(lUmgrenzung);
                    intOBJ_B.Add(lObenB.FindeKnotenNr().Count + z + 1 + lineCount);
                    z++;
                }
                foreach (string l in LlistB) RFEM_COM.CreateQuad(l); 

                // Export Solids
                List<string> Slist = new List<string>();
                z = 0;
                foreach (string e in LlistA)
                {
                    int off1 = 1 + surfaceCount;
                    int off2 = 1 + LlistA.Count + surfaceCount;
                    int off3 = off2 + LlistO.Count;
                    int off4 = off3 + LlistU.Count;
                    int off5 = off4 + 1;
                    int off6 = off4 + LlistZ.Count;
                    string sUmgrenzung = Convert.ToString(off1 + z) + ", " + Convert.ToString(off2 + z) + ", " + Convert.ToString(off3 + z) + ", " + Convert.ToString(off4 + z + (z / abschnittAnzahl)) + ", " + Convert.ToString(off5 + z + (z / abschnittAnzahl)) + ", " + Convert.ToString(off6 + z);
                    Slist.Add(sUmgrenzung);
                    z++;
                }

                z = 0;
                foreach (string l in Slist)
                {
                    RFEM_COM.CreateSolid(l, z+1+solidCount, z+1+surfaceCount+LlistA.Count);
                    z++;
                }
            }
        }


        /// <summary>
        /// Provides an Icon for every component that will be visible in the User Interface.
        /// Icons need to be 24x24 pixels.
        /// </summary>
        protected override System.Drawing.Bitmap Icon
        {
            get
            {
                return rFER_ExcelExp.Properties.Resources.logo;
            }
        }

        /// <summary>
        /// Each component must have a unique Guid to identify it. 
        /// It is vital this Guid doesn't change otherwise old ghx files 
        /// that use the old ID will partially fail during loading.
        /// </summary>
        public override Guid ComponentGuid
        {
            get { return new Guid("34ae557a-e9c4-4ae2-b71d-cf1d3724f70f"); }
        }
    }
}
